/***********************************************************************
 * File: unit_test.c
 * 	(Extracted from
 *	   "$Id: unit_test.c,v 1.1 2008/08/09 19:22:10 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source implements a unit test program for "conv_line_ctrl()".
 *
 * Copyright (C) 2006-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include <limits.h>
#include "conv_line_ctrl.h"

extern int snprintf(char *str, size_t size, const char *format, ...);

static void print_chars(char *sp)
{
	if (sp == NULL) {
		printf("(null)");
	}
	else {
		char	*cp = sp;
		while (*sp != '\0') {
			switch (*sp) {
			case '\n':
				printf("%s\\n", (cp == sp)? "": " ");
				break;
			case '\r':
				printf("%s\\r", (cp == sp)? "": " ");
				break;
			case '\t':
				printf("%s\\t", (cp == sp)? "": " ");
				break;
			case '\v':
				printf("%s\\v", (cp == sp)? "": " ");
				break;
			case '\f':
				printf("%s\\f", (cp == sp)? "": " ");
				break;
			default:
				printf("%s%c"
					, (cp == sp)? "": " ", *sp
					);
				break;
			}
			++sp;
		}
	}
	(void)fflush(stdout);

	return;
}

int main(int ac, char **av)
{
	int	style;
	char	*result, dst[7], *dstp, src[7], *srcp;
	size_t	dst_sz, len, srclen, i1, i2, i3;
	char	*s1[] 	= { "", "\n", "\r\n", "\r", "x" }
		, *s2[]	= { "", "\n", "\r\n", "\r", "y" }
		, *s3[]	= { "", "\n", "\r\n", "\r", "z" }
		;
#	define eval_conv_line_ctrl(STYLE,DSTP,DST_SZ,SRCP) {	\
		char	*seqs[] = { "\r", "\n", "\r\n" };	\
		size_t	seqs_ix;				\
		srclen	= ((SRCP) != NULL)? strlen((SRCP)): 0;	\
		result = conv_line_ctrl((Conv_line_ctrl_style_t)\
					  (STYLE)		\
					, (DSTP)		\
					, (DST_SZ)		\
					, (SRCP)		\
					);			\
		len	= (result != NULL)? strlen(dst): 0;	\
		printf("conv_line_ctrl(style = %d, "		\
			"dst@%p ["				\
			,			(STYLE)		\
			, (DSTP)				\
		); print_chars((DSTP)				\
		); printf("], dst_sz = %lu, src@%p ["		\
			,	(unsigned long)(DST_SZ)		\
			,	(SRCP)				\
		); print_chars((SRCP)				\
		); printf("] (len = %lu))\n"			\
			"...returns @%p ["			\
			,	(unsigned long)srclen		\
			,	result				\
		); print_chars(result				\
		); printf("] (len = %lu) and "			\
			"errno = %d.\n"				\
			,	(unsigned long)len		\
			,	errno				\
		); (void)fflush(stdout);			\
		for (seqs_ix = 0				\
		; seqs_ix < (sizeof(seqs) / sizeof(seqs[0]))	\
		; ++seqs_ix) {					\
		printf("count_lines(src@%p, "			\
			"line_ctrl_sequence@%p=[%s]) "		\
			"returns %lu. (errno=%d)\n"		\
			, (SRCP)				\
			, seqs[seqs_ix], seqs[seqs_ix]		\
			, (unsigned long)count_lines((SRCP)	\
					, seqs[seqs_ix]		\
					)			\
			, errno					\
			); (void)fflush(stdout);		\
		}						\
	}

	printf("{{{ error cases starts (ac=%d, av@%p)...\n"
		,			ac, (void *)av
		); (void)fflush(stdout);

	style	= INT_MIN;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= NULL;
	dst_sz	= 0;
	srcp	= NULL;
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= NULL;
	dst_sz	= 0;
	srcp	= NULL;
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 0;
	srcp	= NULL;
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= sizeof(dst);
	srcp	= NULL;
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Unix;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Mac_to_Dos;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 6;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Unix_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\n\n\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\n\r\n\r\n");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	style	= Conv_line_ctrl_Dos_to_Mac;
	memset(dst, (int)'@', sizeof(dst) - 1)
	; dst[sizeof(dst) - 1] = '\0';
	dstp	= dst;
	dst_sz	= 3;
	srcp	= strcpy(src, "\r\r\r");
	eval_conv_line_ctrl(style, dstp, dst_sz, srcp);

	printf("}}} error cases finished!\n\n"); (void)fflush(stdout);

	for (style = Conv_line_ctrl_to_Unix
	; style <= Conv_line_ctrl_Dos_to_Mac
	; ++style) {
		printf("{{{ `style = %d' case starts...\n", style)
		; (void)fflush(stdout);

		for (i1 = 0; i1 < sizeof(s1) / sizeof(s1[0]); ++i1) {
		for (i2 = 0; i2 < sizeof(s2) / sizeof(s2[0]); ++i2) {
		for (i3 = 0; i3 < sizeof(s3) / sizeof(s3[0]); ++i3) {
			memset(dst, (int)'@', sizeof(dst) - 1);
			dst[sizeof(dst) - 1] = '\0';
			dstp	= dst;
			dst_sz	= sizeof(dst);
			(void)snprintf(src, sizeof(src), "%s%s%s"
				,		s1[i1], s2[i2], s3[i3]
				);
			srcp	= src;
			eval_conv_line_ctrl(style, dstp, dst_sz, srcp);
		}}}
		printf("}}} `style = %d' case finished!.\n\n", style)
		; (void)fflush(stdout);
	}
#	undef eval_conv_line_ctrl

	exit(0);
}
